@@ -118,6 +118,11 @@ namespace Sass {
     pstate_.offset += pstate - pstate_ + pstate.offset;
   }
 
+  void AST_Node::set_pstate_offset(const Offset& offset)
+  {
+    pstate_.offset = offset;
+  }
+
   inline bool is_ns_eq(const std::string& l, const std::string& r)
   {
     if (l.empty() && r.empty()) return true;
@@ -153,12 +158,26 @@ namespace Sass {
     return false;
   }
 
+  bool SimpleSequence_Selector::has_real_parent_ref()
+  {
+    for (auto __s = elements().begin(); __s != elements().end(); ++__s) { Simple_Selector* s = *(__s);
+      if (s && s->has_real_parent_ref()) return true;
+    }
+    return false;
+  }
+
   bool Sequence_Selector::has_parent_ref()
   {
     return (head() && head()->has_parent_ref()) ||
            (tail() && tail()->has_parent_ref());
   }
 
+  bool Sequence_Selector::has_real_parent_ref()
+  {
+    return (head() && head()->has_real_parent_ref()) ||
+           (tail() && tail()->has_real_parent_ref());
+  }
+
   bool Sequence_Selector::operator< (const Sequence_Selector& rhs) const
   {
     // const iterators for tails
@@ -1036,24 +1055,32 @@ namespace Sass {
         size_t i = 0, L = h->length();
         if (dynamic_cast<Element_Selector*>(h->first())) {
           if (Class_Selector* sq = dynamic_cast<Class_Selector*>(rh->last())) {
-            Class_Selector* sqs = new Class_Selector(*sq);
+            Class_Selector* sqs = SASS_MEMORY_NEW(ctx.mem, Class_Selector, *sq);
             sqs->name(sqs->name() + (*h)[0]->name());
+            sqs->pstate((*h)[0]->pstate());
             (*rh)[rh->length()-1] = sqs;
+            rh->pstate(h->pstate());
             for (i = 1; i < L; ++i) *rh << (*h)[i];
           } else if (Id_Selector* sq = dynamic_cast<Id_Selector*>(rh->last())) {
-            Id_Selector* sqs = new Id_Selector(*sq);
+            Id_Selector* sqs = SASS_MEMORY_NEW(ctx.mem, Id_Selector, *sq);
             sqs->name(sqs->name() + (*h)[0]->name());
+            sqs->pstate((*h)[0]->pstate());
             (*rh)[rh->length()-1] = sqs;
+            rh->pstate(h->pstate());
             for (i = 1; i < L; ++i) *rh << (*h)[i];
           } else if (Element_Selector* ts = dynamic_cast<Element_Selector*>(rh->last())) {
-            Element_Selector* tss = new Element_Selector(*ts);
+            Element_Selector* tss = SASS_MEMORY_NEW(ctx.mem, Element_Selector, *ts);
             tss->name(tss->name() + (*h)[0]->name());
+            tss->pstate((*h)[0]->pstate());
             (*rh)[rh->length()-1] = tss;
+            rh->pstate(h->pstate());
             for (i = 1; i < L; ++i) *rh << (*h)[i];
           } else if (Placeholder_Selector* ps = dynamic_cast<Placeholder_Selector*>(rh->last())) {
-            Placeholder_Selector* pss = new Placeholder_Selector(*ps);
+            Placeholder_Selector* pss = SASS_MEMORY_NEW(ctx.mem, Placeholder_Selector, *ps);
             pss->name(pss->name() + (*h)[0]->name());
+            pss->pstate((*h)[0]->pstate());
             (*rh)[rh->length()-1] = pss;
+            rh->pstate(h->pstate());
             for (i = 1; i < L; ++i) *rh << (*h)[i];
           } else {
             *last()->head_ += h;
@@ -1106,6 +1133,12 @@ namespace Sass {
     Sequence_Selector* tail = this->tail();
     SimpleSequence_Selector* head = this->head();
 
+    if (!this->has_real_parent_ref() && !implicit_parent) {
+      CommaSequence_Selector* retval = SASS_MEMORY_NEW(ctx.mem, CommaSequence_Selector, pstate());
+      *retval << this;
+      return retval;
+    }
+
     // first parentize the tail (which may return an expanded list)
     CommaSequence_Selector* tails = tail ? tail->resolve_parent_refs(ctx, parents, implicit_parent) : 0;
 
@@ -1126,8 +1159,19 @@ namespace Sass {
                 Sequence_Selector* ss = this->clone(ctx);
                 ss->tail(t ? t->clone(ctx) : 0);
                 SimpleSequence_Selector* h = head_->clone(ctx);
+                // remove parent selector from sequence
                 if (h->length()) h->erase(h->begin());
                 ss->head(h->length() ? h : 0);
+                // adjust for parent selector (1 char)
+                if (h->length()) {
+                  ParserState state((*h)[0]->pstate());
+                  state.offset.column += 1;
+                  state.column -= 1;
+                  (*h)[0]->pstate(state);
+                }
+                // keep old parser state
+                s->pstate(pstate());
+                // append new tail
                 s->append(ctx, ss);
                 *retval << s;
               }
@@ -1147,10 +1191,21 @@ namespace Sass {
               }
               ss->tail(tail ? tail->clone(ctx) : 0);
               SimpleSequence_Selector* h = head_->clone(ctx);
+              // remove parent selector from sequence
               if (h->length()) h->erase(h->begin());
               ss->head(h->length() ? h : 0);
               // \/ IMO ruby sass bug \/
               ss->has_line_feed(false);
+              // adjust for parent selector (1 char)
+              if (h->length()) {
+                ParserState state((*h)[0]->pstate());
+                state.offset.column += 1;
+                state.column -= 1;
+                (*h)[0]->pstate(state);
+              }
+              // keep old parser state
+              s->pstate(pstate());
+              // append new tail
               s->append(ctx, ss);
               *retval << s;
             }
@@ -1392,6 +1447,14 @@ namespace Sass {
     return false;
   }
 
+  bool CommaSequence_Selector::has_real_parent_ref()
+  {
+    for (auto __s = elements().begin(); __s != elements().end(); ++__s) { Sequence_Selector* s = *(__s);
+      if (s && s->has_real_parent_ref()) return true;
+    }
+    return false;
+  }
+
   bool Selector_Schema::has_parent_ref()
   {
     if (String_Schema* schema = dynamic_cast<String_Schema*>(contents())) {
@@ -1400,6 +1463,15 @@ namespace Sass {
     return false;
   }
 
+  bool Selector_Schema::has_real_parent_ref()
+  {
+    if (String_Schema* schema = dynamic_cast<String_Schema*>(contents())) {
+      Parent_Selector* p = dynamic_cast<Parent_Selector*>(schema->at(0));
+      return schema->length() > 0 && p != NULL && p->is_real_parent_ref();
+    }
+    return false;
+  }
+
   void CommaSequence_Selector::adjust_after_pushing(Sequence_Selector* c)
   {
     // if (c->has_reference())   has_reference(true);
@@ -1516,6 +1588,13 @@ namespace Sass {
     return result;
   }
 
+  SimpleSequence_Selector& SimpleSequence_Selector::operator<<(Simple_Selector* element)
+  {
+    Vectorized<Simple_Selector*>::operator<<(element);
+    pstate_.offset += element->pstate().offset;
+    return *this;
+  }
+
   SimpleSequence_Selector* SimpleSequence_Selector::minus(SimpleSequence_Selector* rhs, Context& ctx)
   {
     SimpleSequence_Selector* result = SASS_MEMORY_NEW(ctx.mem, SimpleSequence_Selector, pstate());
@@ -1611,9 +1690,10 @@ namespace Sass {
   }
 
   bool Ruleset::is_invisible() const {
-    CommaSequence_Selector* sl = static_cast<CommaSequence_Selector*>(selector());
+    if (CommaSequence_Selector* sl = dynamic_cast<CommaSequence_Selector*>(selector())) {
     for (size_t i = 0, L = sl->length(); i < L; ++i)
       if (!(*sl)[i]->has_placeholder()) return false;
+    }
     return true;
   }
 
@@ -2220,4 +2300,20 @@ namespace Sass {
     }
   }
 
+  //////////////////////////////////////////////////////////////////////////////////////////
+  // Convert map to (key, value) list.
+  //////////////////////////////////////////////////////////////////////////////////////////
+  List* Map::to_list(Context& ctx, ParserState& pstate) {
+    List* ret = SASS_MEMORY_NEW(ctx.mem, List, pstate, length(), SASS_COMMA);
+
+    for (auto __key = keys().begin(); __key != keys().end(); ++__key) { auto key = *(__key);
+      List* l = SASS_MEMORY_NEW(ctx.mem, List, pstate, 2);
+      *l << key;
+      *l << at(key);
+      *ret << l;
+    }
+
+    return ret;
+  }
+
 }
