@@ -1722,23 +1722,6 @@ namespace Sass {
       SimpleSequence_Selector* pHead = pIter->head();
 
       if (pHead) {
-        if (seen.find(*pHead) == seen.end()) {
-          for (auto __pSimple = pHead->elements().begin(); __pSimple != pHead->elements().end(); ++__pSimple) { Simple_Selector* pSimple = *(__pSimple);
-            if (Wrapped_Selector* ws = dynamic_cast<Wrapped_Selector*>(pSimple)) {
-              if (CommaSequence_Selector* sl = dynamic_cast<CommaSequence_Selector*>(ws->selector())) {
-                for (auto __cs = sl->elements().begin(); __cs != sl->elements().end(); ++__cs) { Sequence_Selector* cs = *(__cs);
-                  while (cs) {
-                    if (complexSelectorHasExtension(cs, ctx, subset_map, seen)) {
-                      hasExtension = true;
-                      break;
-                    }
-                    cs = cs->tail();
-                  }
-                }
-              }
-            }
-          }
-        }
         SubsetMapEntries entries = subset_map.get_v(pHead->to_str_vec());
         for (auto __ext = entries.begin(); __ext != entries.end(); ++__ext) { ExtensionPair ext = *(__ext);
           // check if both selectors have the same media block parent
@@ -1988,9 +1971,16 @@ namespace Sass {
                       Wrapped_Selector* cpy_ws = SASS_MEMORY_NEW(ctx.mem, Wrapped_Selector, *ws);
                       CommaSequence_Selector* cpy_ws_sl = SASS_MEMORY_NEW(ctx.mem, CommaSequence_Selector, sl->pstate());
                       // remove parent selectors from inner selector
-                      if (ext_cs->first()) {
-                        if (ext_cs->first()->has_wrapped_selector()) {
-                          continue; // ignore this case for now
+                      if (ext_cs->first() && ext_cs->first()->head()->length() > 0) {
+                        Wrapped_Selector* ext_ws = dynamic_cast<Wrapped_Selector*>(ext_cs->first()->head()->first());
+                        if (ext_ws/* && ext_cs->length() == 1*/) {
+                          CommaSequence_Selector* ws_cs = dynamic_cast<CommaSequence_Selector*>(ext_ws->selector());
+                          SimpleSequence_Selector* ws_ss = ws_cs->first()->head();
+                          if (!(
+                            dynamic_cast<Pseudo_Selector*>(ws_ss->first()) ||
+                            dynamic_cast<Element_Selector*>(ws_ss->first()) ||
+                            dynamic_cast<Placeholder_Selector*>(ws_ss->first())
+                          )) continue;
                         }
                         *cpy_ws_sl << ext_cs->first();
                       }
