@@ -876,7 +876,7 @@ namespace Sass {
 
   void Inspect::operator()(Parent_Selector* p)
   {
-    append_string("&");
+    if (p->is_real_parent_ref()) append_string("&");
   }
 
   void Inspect::operator()(Placeholder_Selector* s)
