@@ -39,10 +39,10 @@ namespace Sass {
           // Set the new placeholder selector list
           r->selector(remove_placeholders(sl));
           // Remove placeholders in wrapped selectors
-          for (Sequence_Selector* cs : *sl) {
+          for (auto __cs = sl->elements().begin(); __cs != sl->elements().end(); ++__cs) { Sequence_Selector* cs = *(__cs);
             while (cs) {
               if (cs->head()) {
-                for (Simple_Selector* ss : *cs->head()) {
+                for (auto __ss = cs->head()->elements().begin(); __ss != cs->head()->elements().end(); ++__ss) { Simple_Selector* ss = *(__ss);
                   if (Wrapped_Selector* ws = dynamic_cast<Wrapped_Selector*>(ss)) {
                     if (CommaSequence_Selector* sl = dynamic_cast<CommaSequence_Selector*>(ws->selector())) {
                       CommaSequence_Selector* clean = remove_placeholders(sl);
