@@ -1,3 +1,4 @@
+require 'open3'
 require_relative 'capture_with_timeout'
 
 class EngineAdapter
@@ -28,12 +29,12 @@ class EngineAdapter
   end
 end
 
-class ExecutableEngineAdapater < EngineAdapter
+class ExecutableEngineAdapter < EngineAdapter
   include CaptureWithTimeout
 
   def initialize(command, description = nil)
     @command = command
-    @timeout = 20
+    @timeout = 60
     @description = description || command
   end
 
@@ -53,8 +54,7 @@ class ExecutableEngineAdapater < EngineAdapter
 
 
   def compile(sass_filename, style, precision)
-    cmd = "#{@command} --precision #{precision}"
-    cmd += " -t #{style}" if style
+    cmd = "#{@command} --precision #{precision} -t #{style || "expanded"}"
     result = capture3_with_timeout("#{cmd} #{sass_filename}", :binmode => true, :timeout => @timeout)
 
     if result[:timeout]
@@ -66,16 +66,8 @@ class ExecutableEngineAdapater < EngineAdapter
 end
 
 class SassEngineAdapter < EngineAdapter
-  def initialize(description)
-    @description = description
-  end
-
-  def set_description(description)
-    @description = description
-  end
-
   def describe
-    @description
+    'ruby-sass'
   end
 
   def version
@@ -101,7 +93,7 @@ class SassEngineAdapter < EngineAdapter
       Encoding.default_external = "UTF-8"
       Sass::Script::Value::Number.precision = precision
       sass_options = {}
-      sass_options[:style] = style.to_sym if style
+      sass_options[:style] = (style || :expanded).to_sym
       sass_options[:cache] = false unless sass_options.has_key?(:cache)
       css_output = Sass.compile_file(sass_filename.to_s, sass_options)
       # strings come back as utf8 encoded
@@ -124,3 +116,58 @@ class SassEngineAdapter < EngineAdapter
     Encoding.default_external = encoding
   end
 end
+
+class DartEngineAdapter < EngineAdapter
+  def initialize(path)
+    @path = path
+    Tempfile.open("dart-sass-spec") do |f|
+      f.write(<<-DART)
+        import "dart:convert";
+        import "dart:io";
+
+        import "#{File.absolute_path @path}/bin/sass.dart" as sass;
+
+        main() async {
+          await for (var line in new LineSplitter().bind(UTF8.decoder.bind(stdin))) {
+            try {
+              await sass.main(line.split(" "));
+            } catch (error, stackTrace) {
+              stderr.writeln("Unhandled exception:");
+              stderr.writeln(error);
+              stderr.writeln(stackTrace);
+              exitCode = 255;
+            }
+
+            stdout.writeCharCode(0);
+            stdout.write(exitCode);
+            stdout.writeCharCode(0);
+            stderr.writeCharCode(0);
+            exitCode = 0;
+          }
+        }
+      DART
+      @stdin, @stdout, @stderr = Open3.popen3("dart --packages=#{@path}/.packages #{f.path}")
+    end
+  end
+
+  def describe
+    "dart-sass"
+  end
+
+  def version
+    `dart #{@path}/bin/sass.dart --version`
+  end
+
+  def compile(sass_filename, style, precision)
+    @stdin.puts "--no-color --style #{style || 'expanded'} #{sass_filename}"
+    [next_chunk(@stdout), next_chunk(@stderr), next_chunk(@stdout).to_i]
+  end
+
+  private
+
+  def next_chunk(io)
+    result = io.gets("\0")
+    return '' unless result
+    return result[0...-1].force_encoding('binary')
+  end
+end
